@@ -67,8 +67,11 @@ Get the member args for an operations::
 
 """
 import os
+import sys
+import glob
 import json
 import logging
+from .compat import OrderedDict
 import botocore.exceptions
 
 logger = logging.getLogger(__name__)
@@ -76,22 +79,46 @@ logger = logging.getLogger(__name__)
 _data_cache = {}
 _search_paths = []
 
-
 def _load_data(session, data_path):
     logger.debug('Attempting to Load: %s' % data_path)
     data = {}
     file_name = data_path + '.json'
     for path in get_search_path(session):
         file_path = os.path.join(path, file_name)
-        if os.path.isfile(file_path):
+        dir_path = os.path.join(path, data_path)
+        # Is the path a directory?
+        if os.path.isdir(dir_path):
+            logger.debug('Found data dir: %s' % dir_path)
+            try:
+                data = []
+                for pn in glob.glob(os.path.join(dir_path, '*.json')):
+                    fn = os.path.split(pn)[1]
+                    fn = os.path.splitext(fn)[0]
+                    if not fn.startswith('_'):
+                        data.append(fn)
+            except:
+                logger.error('Unable to load dir: %s' % dir_path,
+                             exc_info=True)
+            break
+        elif os.path.isfile(file_path):
             fp = open(file_path)
             try:
-                new_data = json.load(fp)
+                version_info = sys.version_info
+                if version_info[0] == 2 and version_info[1] == 6:
+                    new_data = json.load(fp)
+                else:
+                    new_data = json.load(fp, object_pairs_hook=OrderedDict)
                 fp.close()
                 logger.debug('Found data file: %s' % file_path)
-                data.update(new_data)
+                if data is not None:
+                    data.update(new_data)
+                else:
+                    data = new_data
             except:
-                logger.error('Unable to load file: %s' % file_path)
+                logger.error('Unable to load file: %s' % file_path,
+                             exc_info=True)
+        else:
+            logger.error('Unable to find file: %s' % file_path)
     if data:
         _data_cache[data_path] = data
     return data
@@ -123,6 +150,10 @@ def _load_nested_data(session, data_path):
                     if 'name' in item and item['name'] == attr:
                         data = item
                         break
+        # If we have gotten here and the data is still the original
+        # prefix data, it means we did not find the sub data.
+        if data == _data_cache[prefix]:
+            data = None
     if data is not None:
         _data_cache[data_path] = data
     return data
