@@ -126,7 +126,8 @@ class ValidationError(BotoCoreError):
     :ivar value: The value that was being validated.
     :ivar type_name: The name of the underlying type.
     """
-    fmt = 'Unable to convert value ({value}) to type {type_name}'
+    fmt = ('Unable to convert value ({value}) to type {type_name} '
+           'for param {param}')
 
 
 class RangeError(BotoCoreError):
@@ -151,3 +152,12 @@ class UnknownServiceStyle(BotoCoreError):
 
 class PaginationError(BotoCoreError):
     fmt = 'Error during pagination: {message}'
+
+
+class EventNotFound(BotoCoreError):
+    """
+    The specified event name is unknown to the system.
+
+    :ivar event_name: The name of the event the user attempted to use.
+    """
+    fmt = 'The event ({event_name}) is not known'
