@@ -38,6 +38,10 @@ class Operation(BotoCoreObject):
         self.output = {}
         BotoCoreObject.__init__(self, **op_data)
         self.service = service
+        if self.service:
+            self.session = self.service.session
+        else:
+            self.session = None
         self.type = 'operation'
         self._get_parameters()
         if paginator_cls is None:
@@ -48,9 +52,21 @@ class Operation(BotoCoreObject):
         return 'Operation:%s' % self.name
 
     def call(self, endpoint, **kwargs):
-        logger.debug(kwargs)
+        logger.debug("%s called with kwargs: %s", self, kwargs)
+        event = self.session.create_event('before-call',
+                                          self.service.endpoint_prefix,
+                                          self.name)
         params = self.build_parameters(**kwargs)
-        return endpoint.make_request(self, params)
+        self.session.emit(event, operation=self, endpoint=endpoint,
+                          params=params)
+        response = endpoint.make_request(self, params)
+        event = self.session.create_event('after-call',
+                                          self.service.endpoint_prefix,
+                                          self.name)
+        self.session.emit(event, operation=self,
+                          http_response=response[0],
+                          parsed=response[1])
+        return response
 
     @property
     def can_paginate(self):
