@@ -63,14 +63,14 @@ class Service(object):
 
     @property
     def region_names(self):
-        return self.regions.keys()
+        return self.metadata['regions'].keys()
 
-    def build_endpoint_url(self, host, is_secure):
+    def _build_endpoint_url(self, host, is_secure):
         if is_secure:
             scheme = 'https'
         else:
             scheme = 'http'
-        if scheme not in self.protocols:
+        if scheme not in self.metadata['protocols']:
             raise ValueError('Unsupported protocol: %s' % scheme)
         endpoint_url = '%s://%s%s' % (scheme, host, self.path)
         if self.port:
@@ -93,23 +93,23 @@ class Service(object):
         :param endpoint_url: You can explicitly override the default
             computed endpoint name with this parameter.
         """
-        if region_name is None:
-            if self.global_endpoint and not self.session.profile:
-                endpoint_url = self.build_endpoint_url(self.global_endpoint,
-                                                       is_secure)
+        if not region_name:
+            region_name = self.session.get_variable('region')
+            if region_name is None and not self.global_endpoint:
+                envvar_name = self.session.env_vars['region'][1]
+                raise NoRegionError(env_var=envvar_name)
+        if region_name not in self.metadata['regions']:
+            if self.global_endpoint:
+                endpoint_url = self._build_endpoint_url(self.global_endpoint,
+                                                        is_secure)
                 region_name = 'us-east-1'
             else:
-                region_name = self.session.get_variable('region')
-        if region_name is None:
-            envvar_name = self.session.env_vars['region'][1]
-            raise NoRegionError(env_var=envvar_name)
-        if region_name not in self.regions:
-            raise ServiceNotInRegionError(service_name=self.endpoint_prefix,
-                                          region_name=region_name)
-        endpoint_url = endpoint_url or self.regions[region_name]
+                raise ServiceNotInRegionError(service_name=self.endpoint_prefix,
+                                              region_name=region_name)
+        endpoint_url = endpoint_url or self.metadata['regions'][region_name]
         if endpoint_url is None:
             host = '%s.%s.amazonaws.com' % (self.endpoint_prefix, region_name)
-            endpoint_url = self.build_endpoint_url(host, is_secure)
+            endpoint_url = self._build_endpoint_url(host, is_secure)
         return get_endpoint(self, region_name, endpoint_url)
 
     def get_operation(self, operation_name):
