@@ -29,18 +29,35 @@ import logging
 import platform
 import os
 import copy
+import shlex
+
 import botocore.config
 import botocore.credentials
 import botocore.base
 import botocore.service
-from . import __version__
+from botocore.exceptions import ConfigNotFound, EventNotFound
+from botocore.hooks import HierarchicalEmitter
+from botocore import __version__
+from botocore import handlers
+
+
+AllEvents = {
+    'after-call': '.%s.%s',
+    'after-parsed': '.%s.%s.%s.%s',
+    'before-call': '.%s.%s',
+    'service-created': ''
+    }
+"""
+A dictionary where each key is an event name and the value
+is the formatting string used to construct a new event.
+"""
 
 
 EnvironmentVariables = {
     'profile': (None, 'BOTO_DEFAULT_PROFILE', 'default'),
     'region': ('region', 'BOTO_DEFAULT_REGION', None),
     'data_path': ('data_path', 'BOTO_DATA_PATH', None),
-    'config_file': (None, 'AWS_CONFIG_FILE', None),
+    'config_file': (None, 'AWS_CONFIG_FILE', '~/.aws/config'),
     'access_key': ('aws_access_key_id', 'AWS_ACCESS_KEY_ID', None),
     'secret_key': ('aws_secret_access_key', 'AWS_SECRET_ACCESS_KEY', None),
     'token': ('aws_security_token', 'AWS_SECURITY_TOKEN', None),
@@ -72,6 +89,11 @@ The ``profile`` and ``config_file`` variables should always have a
 None value for the first entry in the tuple because it doesn't make
 sense to look inside the config file for the location of the config
 file or for the default profile to use.
+
+The ``config_name`` is the name to look for in the configuration file,
+the ``env var`` is the OS environment variable (``os.environ``) to
+use, and ``default_value`` is the value to use if no value is otherwise
+found.
 """
 
 
@@ -88,7 +110,8 @@ class Session(object):
 
     FmtString = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
 
-    def __init__(self, env_vars=None):
+    def __init__(self, env_vars=None, event_hooks=None,
+                 include_builtin_handlers=True):
         """
         Create a new Session object.
 
@@ -96,25 +119,62 @@ class Session(object):
         :param env_vars: A dictionary that is used to override some or all
             of the environment variables associated with this session.  The
             key/value pairs defined in this dictionary will override the
-            corresponding variables defined in `EnvironmentVariables`.
+            corresponding variables defined in ``EnvironmentVariables``.
+
+        :type event_hooks: BaseEventHooks
+        :param event_hooks: The event hooks object to use. If one is not
+            provided, an event hooks object will be automatically created
+            for you.
+
+        :type include_builtin_handlers: bool
+        :param include_builtin_handlers: Indicates whether or not to
+            automatically register builtin handlers.
         """
         self.env_vars = copy.copy(EnvironmentVariables)
         if env_vars:
             self.env_vars.update(env_vars)
+        if event_hooks is None:
+            self._events = HierarchicalEmitter()
+        else:
+            self._events = event_hooks
+        if include_builtin_handlers:
+            self._register_builtin_handlers(self._events)
         self.user_agent_name = 'Boto'
         self.user_agent_version = __version__
         self._profile = None
         self._config = None
         self._credentials = None
+        self._profile_map = None
+
+    def _register_builtin_handlers(self, events):
+        for event_name, handler in handlers.BUILTIN_HANDLERS:
+            self.register(event_name, handler)
 
     @property
     def available_profiles(self):
-        profiles = []
-        self.get_config()
-        for key in self._config.keys():
-            if not key.startswith('_'):
-                profiles.append(key)
-        return profiles
+        return list(self._build_profile_map().keys())
+
+    def _build_profile_map(self):
+        # This will build the profile map if it has not been created,
+        # otherwise it will return the cached value.  The profile map
+        # is a list of profile names, to the config values for the profile.
+        if self._profile_map is None:
+            profile_map = {}
+            for key, values in self.full_config.items():
+                if key.startswith("profile"):
+                    try:
+                        parts = shlex.split(key)
+                    except ValueError:
+                        continue
+                    if len(parts) == 2:
+                        profile_map[parts[1]] = values
+                elif key == 'default':
+                    # default section is special and is considered a profile
+                    # name but we don't require you use 'profile "default"'
+                    # as a section.
+                    profile_map[key] = values
+            self._profile_map = profile_map
+        return self._profile_map
 
     @property
     def profile(self):
@@ -159,6 +219,8 @@ class Session(object):
                 if config_name:
                     config = self.get_config()
                     value = config.get(config_name, default)
+        if value is None and default is not None:
+            value = default
         return value
 
     def get_config(self):
@@ -168,14 +230,18 @@ class Session(object):
         using the default ``profile`` session variable.  If it has already been
         loaded, the cached configuration will be returned.
 
+        Note that this configuration is specific to a single profile (the
+        ``profile`` session variable).
+
+
         :raises: ConfigNotFound, ConfigParseError
+        :rtype: dict
         """
-        if self._config is None:
-            self._config = botocore.config.get_config(self)
+        config = self.full_config
         profile_name = self.get_variable('profile')
         if not profile_name:
             profile_name = 'default'
-        return self._config.get(profile_name, dict())
+        return self._build_profile_map().get(profile_name, {})
 
     @property
     def full_config(self):
@@ -184,9 +250,14 @@ class Session(object):
         The ``get_config`` method returns the config associated with the
         specified profile.  This property returns the contents of the
         **entire** config file.
+
+        :rtype: dict
         """
         if self._config is None:
-            self._config = botocore.config.get_config(self)
+            try:
+                self._config = botocore.config.get_config(self)
+            except ConfigNotFound:
+                self._config = {}
         return self._config
 
     def set_credentials(self, access_key, secret_key, token=None):
@@ -265,30 +336,20 @@ class Session(object):
         """
         return botocore.base.get_data(self, data_path)
 
-    def get_service_metadata(self, service_name, provider_name='aws'):
-        """
-        Get metadata for a service.
-        """
-        data_path = '%s/_services/%s' % (provider_name, service_name)
-        return self.get_data(data_path)
-
     def get_service_data(self, service_name, provider_name='aws'):
         """
         Retrieve the fully merged data associated with a service.
         """
-        meta_data = self.get_service_metadata(service_name, provider_name)
         data_path = '%s/%s' % (provider_name, service_name)
         service_data = self.get_data(data_path)
-        # Merge metadata about service
-        service_data.update(meta_data)
         return service_data
 
     def get_available_services(self, provider_name='aws'):
         """
         Return a list of names of available services.
         """
-        data_path = '%s/_services' % provider_name
-        return self.get_data(data_path).keys()
+        data_path = '%s' % provider_name
+        return self.get_data(data_path)
 
     def get_service(self, service_name, provider_name='aws'):
         """
@@ -303,7 +364,11 @@ class Session(object):
 
         :returns: :class:`botocore.service.Service`
         """
-        return botocore.service.get_service(self, service_name, provider_name)
+        service = botocore.service.get_service(self, service_name,
+                                               provider_name)
+        event = self.create_event('service-created')
+        self._events.emit(event, service=service)
+        return service
 
     def set_debug_logger(self):
         """
@@ -354,9 +419,63 @@ class Session(object):
         # add ch to logger
         log.addHandler(ch)
 
+    def register(self, event_name, handler):
+        """Register a handler with an event.
+
+        :type event_name: str
+        :param event_name: The name of the event.
+
+        :type handler: callable
+        :param handler: The callback to invoke when the event
+            is emitted.  This object must be callable, and must
+            accept ``**kwargs``.  If either of these preconditions are
+            not met, a ``ValueError`` will be raised.
+
+        """
+        self._events.register(event_name, handler)
+
+    def unregister(self, event_name, handler):
+        """Unregister a handler with an event.
+
+        :type event_name: str
+        :param event_name: The name of the event.
+
+        :type handler: callable
+        :param handler: The callback to unregister.
+
+        """
+        self._events.unregister(event_name, handler)
+
+    def create_event(self, event_name, *fmtargs):
+        """
+        Creates a new event string that can then be emitted.
+        You could just create it manually, since it's just
+        a string but this helps to define the range of known events.
+
+        :type event_name: str
+        :param event_name: The base name of the new event.
+
+        :type fmtargs: tuple
+        :param fmtargs: A tuple of values that will be used as the
+            arguments pass to the string formatting operation.  The
+            actual values passed depend on the type of event you
+            are creating.
+        """
+        if event_name in AllEvents:
+            fmt_string = AllEvents[event_name]
+            if fmt_string:
+                event = event_name + (fmt_string % fmtargs)
+            else:
+                event = event_name
+            return event
+        raise EventNotFound(event_name=event_name)
+
+    def emit(self, event_name, **kwargs):
+        return self._events.emit(event_name, **kwargs)
+
 
 def get_session(env_vars=None):
     """
-    Return a new session  object.
+    Return a new session object.
     """
     return Session(env_vars)
